@@ -5,7 +5,7 @@
     Test message patching for internationalization purposes.  Runs the text
     builder in the test root.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,14 +13,14 @@ import os
 import re
 
 import pytest
-from babel.messages import pofile, mofile
+from babel.messages import mofile, pofile
+from babel.messages.catalog import Catalog
 from docutils import nodes
 
-from sphinx.testing.util import (
-    path, etree_parse, strip_escseq,
-    assert_re_search, assert_not_re_search, assert_startswith, assert_node
-)
-
+from sphinx import locale
+from sphinx.testing.util import (assert_node, assert_not_re_search, assert_re_search,
+                                 assert_startswith, etree_parse, path, strip_escseq)
+from sphinx.util import docutils
 
 sphinx_intl = pytest.mark.sphinx(
     testroot='intl',
@@ -90,15 +90,6 @@ def assert_count(expected_expr, result, count):
     assert len(re.findall(*find_pair)) == count, find_pair
 
 
-@sphinx_intl
-@pytest.mark.sphinx('text')
-@pytest.mark.test_params(shared_result='test_intl_basic')
-def test_text_toctree(app):
-    app.build()
-    result = (app.outdir / 'index.txt').text()
-    assert_startswith(result, "CONTENTS\n********\n\nTABLE OF CONTENTS\n")
-
-
 @sphinx_intl
 @pytest.mark.sphinx('text')
 @pytest.mark.test_params(shared_result='test_intl_basic')
@@ -117,7 +108,7 @@ def test_text_emit_warnings(app, warning):
 def test_text_warning_node(app):
     app.build()
     # test warnings in translation
-    result = (app.outdir / 'warnings.txt').text()
+    result = (app.outdir / 'warnings.txt').read_text()
     expect = ("3. I18N WITH REST WARNINGS"
               "\n**************************\n"
               "\nLINE OF >>``<<BROKEN LITERAL MARKUP.\n")
@@ -131,7 +122,7 @@ def test_text_warning_node(app):
 def test_text_title_underline(app):
     app.build()
     # --- simple translation; check title underlines
-    result = (app.outdir / 'bom.txt').text()
+    result = (app.outdir / 'bom.txt').read_text()
     expect = ("2. Datei mit UTF-8"
               "\n******************\n"  # underline matches new translation
               "\nThis file has umlauts: äöü.\n")
@@ -144,7 +135,7 @@ def test_text_title_underline(app):
 def test_text_subdirs(app):
     app.build()
     # --- check translation in subdirs
-    result = (app.outdir / 'subdir' / 'index.txt').text()
+    result = (app.outdir / 'subdir' / 'index.txt').read_text()
     assert_startswith(result, "1. subdir contents\n******************\n")
 
 
@@ -154,7 +145,7 @@ def test_text_subdirs(app):
 def test_text_inconsistency_warnings(app, warning):
     app.build()
     # --- check warnings for inconsistency in number of references
-    result = (app.outdir / 'refs_inconsistency.txt').text()
+    result = (app.outdir / 'refs_inconsistency.txt').read_text()
     expect = ("8. I18N WITH REFS INCONSISTENCY"
               "\n*******************************\n"
               "\n* FOR CITATION [ref3].\n"
@@ -204,7 +195,7 @@ def test_text_inconsistency_warnings(app, warning):
 def test_text_literalblock_warnings(app, warning):
     app.build()
     # --- check warning for literal block
-    result = (app.outdir / 'literalblock.txt').text()
+    result = (app.outdir / 'literalblock.txt').read_text()
     expect = ("9. I18N WITH LITERAL BLOCK"
               "\n**************************\n"
               "\nCORRECT LITERAL BLOCK:\n"
@@ -226,7 +217,7 @@ def test_text_literalblock_warnings(app, warning):
 def test_text_definition_terms(app):
     app.build()
     # --- definition terms: regression test for #975, #2198, #2205
-    result = (app.outdir / 'definition_terms.txt').text()
+    result = (app.outdir / 'definition_terms.txt').read_text()
     expect = ("13. I18N WITH DEFINITION TERMS"
               "\n******************************\n"
               "\nSOME TERM"
@@ -246,7 +237,7 @@ def test_text_definition_terms(app):
 def test_text_glossary_term(app, warning):
     app.build()
     # --- glossary terms: regression test for #1090
-    result = (app.outdir / 'glossary_terms.txt').text()
+    result = (app.outdir / 'glossary_terms.txt').read_text()
     expect = ("18. I18N WITH GLOSSARY TERMS"
               "\n****************************\n"
               "\nSOME NEW TERM"
@@ -265,7 +256,7 @@ def test_text_glossary_term(app, warning):
 def test_text_glossary_term_inconsistencies(app, warning):
     app.build()
     # --- glossary term inconsistencies: regression test for #1090
-    result = (app.outdir / 'glossary_terms_inconsistency.txt').text()
+    result = (app.outdir / 'glossary_terms_inconsistency.txt').read_text()
     expect = ("19. I18N WITH GLOSSARY TERMS INCONSISTENCY"
               "\n******************************************\n"
               "\n1. LINK TO *SOME NEW TERM*.\n")
@@ -298,7 +289,7 @@ def test_gettext_section(app):
 def test_text_section(app):
     app.build()
     # --- section
-    result = (app.outdir / 'section.txt').text()
+    result = (app.outdir / 'section.txt').read_text()
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'section.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
@@ -310,7 +301,7 @@ def test_text_section(app):
 def test_text_seealso(app):
     app.build()
     # --- seealso
-    result = (app.outdir / 'seealso.txt').text()
+    result = (app.outdir / 'seealso.txt').read_text()
     expect = ("12. I18N WITH SEEALSO"
               "\n*********************\n"
               "\nSee also: SHORT TEXT 1\n"
@@ -327,7 +318,7 @@ def test_text_seealso(app):
 def test_text_figure_captions(app):
     app.build()
     # --- figure captions: regression test for #940
-    result = (app.outdir / 'figure.txt').text()
+    result = (app.outdir / 'figure.txt').read_text()
     expect = ("14. I18N WITH FIGURE CAPTION"
               "\n****************************\n"
               "\n   [image]MY CAPTION OF THE FIGURE\n"
@@ -345,9 +336,9 @@ def test_text_figure_captions(app):
               "14.2. IMAGE URL AND ALT\n"
               "=======================\n"
               "\n"
-              "[image: i18n][image]\n"
+              "[image: I18N -> IMG][image]\n"
               "\n"
-              "   [image: img][image]\n"
+              "   [image: IMG -> I18N][image]\n"
               "\n"
               "\n"
               "14.3. IMAGE ON SUBSTITUTION\n"
@@ -371,7 +362,7 @@ def test_text_figure_captions(app):
 def test_text_rubric(app):
     app.build()
     # --- rubric: regression test for pull request #190
-    result = (app.outdir / 'rubric.txt').text()
+    result = (app.outdir / 'rubric.txt').read_text()
     expect = ("I18N WITH RUBRIC"
               "\n****************\n"
               "\n-[ RUBRIC TITLE ]-\n"
@@ -389,7 +380,7 @@ def test_text_rubric(app):
 def test_text_docfields(app):
     app.build()
     # --- docfields
-    result = (app.outdir / 'docfields.txt').text()
+    result = (app.outdir / 'docfields.txt').read_text()
     expect = ("21. I18N WITH DOCFIELDS"
               "\n***********************\n"
               "\nclass Cls1\n"
@@ -420,7 +411,7 @@ def test_text_admonitions(app):
     # --- admonitions
     # #1206: gettext did not translate admonition directive's title
     # seealso: http://docutils.sourceforge.net/docs/ref/rst/directives.html#admonitions
-    result = (app.outdir / 'admonitions.txt').text()
+    result = (app.outdir / 'admonitions.txt').read_text()
     directives = (
         "attention", "caution", "danger", "error", "hint",
         "important", "note", "tip", "warning", "admonition")
@@ -437,11 +428,16 @@ def test_text_admonitions(app):
 @pytest.mark.test_params(shared_result='test_intl_gettext')
 def test_gettext_toctree(app):
     app.build()
-    # --- toctree
+    # --- toctree (index.rst)
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'index.po')
     actual = read_po(app.outdir / 'index.pot')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.id in [m.id for m in actual if m.id]
+    # --- toctree (toctree.rst)
+    expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'toctree.po')
+    actual = read_po(app.outdir / 'toctree.pot')
+    for expect_msg in [m for m in expect if m.id]:
+        assert expect_msg.id in [m.id for m in actual if m.id]
 
 
 @sphinx_intl
@@ -462,31 +458,24 @@ def test_gettext_table(app):
 def test_text_table(app):
     app.build()
     # --- toctree
-    result = (app.outdir / 'table.txt').text()
+    result = (app.outdir / 'table.txt').read_text()
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'table.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
 
 
-@sphinx_intl
-@pytest.mark.sphinx('gettext')
-@pytest.mark.test_params(shared_result='test_intl_gettext')
-def test_gettext_toctree(app):
-    app.build()
-    # --- toctree
-    expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'toctree.po')
-    actual = read_po(app.outdir / 'toctree.pot')
-    for expect_msg in [m for m in expect if m.id]:
-        assert expect_msg.id in [m.id for m in actual if m.id]
-
-
 @sphinx_intl
 @pytest.mark.sphinx('text')
 @pytest.mark.test_params(shared_result='test_intl_basic')
 def test_text_toctree(app):
     app.build()
-    # --- toctree
-    result = (app.outdir / 'toctree.txt').text()
+    # --- toctree (index.rst)
+    # Note: index.rst contains contents that is not shown in text.
+    result = (app.outdir / 'index.txt').read_text()
+    assert 'CONTENTS' in result
+    assert 'TABLE OF CONTENTS' in result
+    # --- toctree (toctree.rst)
+    result = (app.outdir / 'toctree.txt').read_text()
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'toctree.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
@@ -510,7 +499,7 @@ def test_gettext_topic(app):
 def test_text_topic(app):
     app.build()
     # --- topic
-    result = (app.outdir / 'topic.txt').text()
+    result = (app.outdir / 'topic.txt').read_text()
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'topic.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
@@ -628,7 +617,7 @@ def test_gettext_dont_rebuild_mo(make_app, app_params):
 def test_html_meta(app):
     app.build()
     # --- test for meta
-    result = (app.outdir / 'index.html').text()
+    result = (app.outdir / 'index.html').read_text()
     expected_expr = '<meta content="TESTDATA FOR I18N" name="description" />'
     assert expected_expr in result
     expected_expr = '<meta content="I18N, SPHINX, MARKUP" name="keywords" />'
@@ -644,7 +633,7 @@ def test_html_footnotes(app):
     app.build()
     # --- test for #955 cant-build-html-with-footnotes-when-using
     # expect no error by build
-    (app.outdir / 'footnote.html').text()
+    (app.outdir / 'footnote.html').read_text()
 
 
 @sphinx_intl
@@ -653,7 +642,7 @@ def test_html_footnotes(app):
 def test_html_undefined_refs(app):
     app.build()
     # --- links to undefined reference
-    result = (app.outdir / 'refs_inconsistency.html').text()
+    result = (app.outdir / 'refs_inconsistency.html').read_text()
 
     expected_expr = ('<a class="reference external" '
                      'href="http://www.example.com">reference</a>')
@@ -675,7 +664,7 @@ def test_html_undefined_refs(app):
 def test_html_index_entries(app):
     app.build()
     # --- index entries: regression test for #976
-    result = (app.outdir / 'genindex.html').text()
+    result = (app.outdir / 'genindex.html').read_text()
 
     def wrap(tag, keyword):
         start_tag = "<%s[^>]*>" % tag
@@ -713,7 +702,7 @@ def test_html_index_entries(app):
 def test_html_versionchanges(app):
     app.build()
     # --- versionchanges
-    result = (app.outdir / 'versionchange.html').text()
+    result = (app.outdir / 'versionchange.html').read_text()
 
     def get_content(result, name):
         matched = re.search(r'<div class="%s">\n*(.*?)</div>' % name,
@@ -750,7 +739,7 @@ def test_html_docfields(app):
     app.build()
     # --- docfields
     # expect no error by build
-    (app.outdir / 'docfields.html').text()
+    (app.outdir / 'docfields.html').read_text()
 
 
 @sphinx_intl
@@ -759,7 +748,7 @@ def test_html_docfields(app):
 def test_html_template(app):
     app.build()
     # --- gettext template
-    result = (app.outdir / 'contents.html').text()
+    result = (app.outdir / 'contents.html').read_text()
     assert "WELCOME" in result
     assert "SPHINX 2013.120" in result
 
@@ -946,19 +935,19 @@ def test_xml_role_xref(app):
         ['LINK TO', "I18N ROCK'N ROLE XREF", ',', 'CONTENTS', ',',
          'SOME NEW TERM', '.'],
         ['i18n-role-xref', 'index',
-         'glossary_terms#term-some-term'])
+         'glossary_terms#term-Some-term'])
 
     para2 = sec2.findall('paragraph')
     assert_elem(
         para2[0],
         ['LINK TO', 'SOME OTHER NEW TERM', 'AND', 'SOME NEW TERM', '.'],
-        ['glossary_terms#term-some-other-term',
-         'glossary_terms#term-some-term'])
+        ['glossary_terms#term-Some-other-term',
+         'glossary_terms#term-Some-term'])
     assert_elem(
         para2[1],
-        ['LINK TO', 'SAME TYPE LINKS', 'AND',
-         "I18N ROCK'N ROLE XREF", '.'],
-        ['same-type-links', 'i18n-role-xref'])
+        ['LINK TO', 'LABEL', 'AND',
+         'SAME TYPE LINKS', 'AND', 'SAME TYPE LINKS', '.'],
+        ['i18n-role-xref', 'same-type-links', 'same-type-links'])
     assert_elem(
         para2[2],
         ['LINK TO', 'I18N WITH GLOSSARY TERMS', 'AND', 'CONTENTS', '.'],
@@ -1056,7 +1045,7 @@ def test_xml_label_targets(app):
 def test_additional_targets_should_not_be_translated(app):
     app.build()
     # [literalblock.txt]
-    result = (app.outdir / 'literalblock.html').text()
+    result = (app.outdir / 'literalblock.html').read_text()
 
     # title should be translated
     expected_expr = 'CODE-BLOCKS'
@@ -1092,22 +1081,26 @@ def test_additional_targets_should_not_be_translated(app):
 
     # [raw.txt]
 
-    result = (app.outdir / 'raw.html').text()
+    result = (app.outdir / 'raw.html').read_text()
 
     # raw block should not be translated
-    expected_expr = """<iframe src="http://sphinx-doc.org"></iframe></div>"""
-    assert_count(expected_expr, result, 1)
+    if docutils.__version_info__ < (0, 17):
+        expected_expr = """<iframe src="http://sphinx-doc.org"></iframe></div>"""
+        assert_count(expected_expr, result, 1)
+    else:
+        expected_expr = """<iframe src="http://sphinx-doc.org"></iframe></section>"""
+        assert_count(expected_expr, result, 1)
 
     # [figure.txt]
 
-    result = (app.outdir / 'figure.html').text()
+    result = (app.outdir / 'figure.html').read_text()
 
-    # alt and src for image block should not be translated
-    expected_expr = """<img alt="i18n" src="_images/i18n.png" />"""
+    # src for image block should not be translated (alt is translated)
+    expected_expr = """<img alt="I18N -&gt; IMG" src="_images/i18n.png" />"""
     assert_count(expected_expr, result, 1)
 
-    # alt and src for figure block should not be translated
-    expected_expr = """<img alt="img" src="_images/img.png" />"""
+    # src for figure block should not be translated (alt is translated)
+    expected_expr = """<img alt="IMG -&gt; I18N" src="_images/img.png" />"""
     assert_count(expected_expr, result, 1)
 
 
@@ -1130,7 +1123,7 @@ def test_additional_targets_should_not_be_translated(app):
 def test_additional_targets_should_be_translated(app):
     app.build()
     # [literalblock.txt]
-    result = (app.outdir / 'literalblock.html').text()
+    result = (app.outdir / 'literalblock.html').read_text()
 
     # title should be translated
     expected_expr = 'CODE-BLOCKS'
@@ -1166,15 +1159,19 @@ def test_additional_targets_should_be_translated(app):
 
     # [raw.txt]
 
-    result = (app.outdir / 'raw.html').text()
+    result = (app.outdir / 'raw.html').read_text()
 
     # raw block should be translated
-    expected_expr = """<iframe src="HTTP://SPHINX-DOC.ORG"></iframe></div>"""
-    assert_count(expected_expr, result, 1)
+    if docutils.__version_info__ < (0, 17):
+        expected_expr = """<iframe src="HTTP://SPHINX-DOC.ORG"></iframe></div>"""
+        assert_count(expected_expr, result, 1)
+    else:
+        expected_expr = """<iframe src="HTTP://SPHINX-DOC.ORG"></iframe></section>"""
+        assert_count(expected_expr, result, 1)
 
     # [figure.txt]
 
-    result = (app.outdir / 'figure.html').text()
+    result = (app.outdir / 'figure.html').read_text()
 
     # alt and src for image block should be translated
     expected_expr = """<img alt="I18N -&gt; IMG" src="_images/img.png" />"""
@@ -1289,3 +1286,30 @@ def test_image_glob_intl_using_figure_language_filename(app):
 
 def getwarning(warnings):
     return strip_escseq(warnings.getvalue().replace(os.sep, '/'))
+
+
+@pytest.mark.sphinx('html', testroot='basic', confoverrides={'language': 'de'})
+def test_customize_system_message(make_app, app_params, sphinx_test_tempdir):
+    try:
+        # clear translators cache
+        locale.translators.clear()
+
+        # prepare message catalog (.po)
+        locale_dir = sphinx_test_tempdir / 'basic' / 'locales' / 'de' / 'LC_MESSAGES'
+        locale_dir.makedirs()
+        with (locale_dir / 'sphinx.po').open('wb') as f:
+            catalog = Catalog()
+            catalog.add('Quick search', 'QUICK SEARCH')
+            pofile.write_po(f, catalog)
+
+        # construct application and convert po file to .mo
+        args, kwargs = app_params
+        app = make_app(*args, **kwargs)
+        assert (locale_dir / 'sphinx.mo').exists()
+        assert app.translator.gettext('Quick search') == 'QUICK SEARCH'
+
+        app.build()
+        content = (app.outdir / 'index.html').read_text()
+        assert 'QUICK SEARCH' in content
+    finally:
+        locale.translators.clear()
